# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2020 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import json
import os
import time
import os.path

from cloudinit.settings import PER_INSTANCE
from cloudinit import util
from cloudinit import netinfo

frequency = PER_INSTANCE

PIDOF = 'pidof'
RMCCTRL = '/usr/sbin/rsct/bin/rmcctrl'
RECFGCT = '/usr/sbin/rsct/install/bin/recfgct'
SRCMSTR = '/sbin/srcmstr'

NODE_ID_FILE = '/etc/ct_node_id'

SRCMSTR_TIMEOUT_SECONDS = 30


def handle(name, _cfg, _cloud, log, _args):
    global RMCCTRL, RECFGCT
    # IBM starts - RTC 76090
    # Skipping reset_rmc when datasource.dsname is None
    # or vopt get removed
    # As recommended the ct_node_id must be unique and should be change
    # Restricting change of ct_node_id
    with open('/run/cloud-init/instance-data.json') as data_file:
        ds = json.load(data_file)
        ds_value = ds['v1'].get('platform')
    if ds_value == 'none':
        log.debug('Skipping creation of new ct_node_id node')
        return
    # IBM ends
    required_tools = [RMCCTRL, RECFGCT]
    for tool in required_tools:
        if not os.path.isfile(tool):
            log.debug('{} is not found, therefore trying {}.'
                      .format(tool, tool.replace('/usr/sbin', '/opt')))
            tool = tool.replace('/usr/sbin', '/opt')
            if tool.split('/')[-1] == 'rmcctrl':
                RMCCTRL = tool
            if tool.split('/')[-1] == 'recfgct':
                RECFGCT = tool
            if not os.path.isfile(tool):
                log.debug('%s is not found but is required, therefore not '
                          'attempting to reset RMC.' % tool)
                return

    # IBM start - RTC 74794
    log.debug('Bringing up the interfaces.')
    info = netinfo.netdev_info()
    info_len = len(info.keys())
    # Trying to make the interfaces up
    for info_count in info.keys():
        try:
            if 'lo' in info_count:
                log.debug('Skipping localhost interface')
                continue
            interface_file = '/etc/sysconfig/network/ifcfg-'+info_count
            util.subp(['ifup', info_count])
        except Exception as e:
            util.logexc(log, 'Interface is not up %s' % e)
            pass
    # IBM ends
    log.debug('Attempting to reset RMC.')

    node_id_before = get_node_id(log)
    log.debug('Node ID at beginning of module: %s' % node_id_before)

    # Stop the RMC subsystem and all resource managers so that we can make
    # some changes to it
    try:
        util.subp([RMCCTRL, '-z'])
    except Exception as e:
        util.logexc(log, 'Failed to stop the RMC subsystem.')
        raise

    reconfigure_rsct_subsystems(log)

    node_id_after = get_node_id(log)
    log.debug('Node ID at end of module: %s' % node_id_after)

    if node_id_after == node_id_before:
        msg = 'New node ID did not get generated.'
        log.error(msg)
        raise Exception(msg)


def reconfigure_rsct_subsystems(log):
    # Reconfigure the RSCT subsystems, which includes removing all RSCT data
    # under the /var/ct directory, generating a new node ID, and making it
    # appear as if the RSCT components were just installed
    try:
        out = util.subp([RECFGCT])[0]
        log.debug(out.strip())
    except util.ProcessExecutionError:
        util.logexc(log, 'Failed to reconfigure the RSCT subsystems.')
        raise


def get_node_id(log):
    try:
        node_id = util.subp(['/usr/bin/head', '-n1', NODE_ID_FILE])[0].strip()
        return node_id
    except util.ProcessExecutionError:
        util.logexc(log, 'Failed to get node ID from file %s.' % NODE_ID_FILE)
        raise

